@@ -309,6 +309,32 @@ webkit_pref_callback_font_family (GSettings *settings,
   g_free (value);
 }
 
+static char **
+normalize_languages (char **languages)
+{
+  int i;
+  GPtrArray *langs;
+
+  langs = g_ptr_array_new ();
+
+  for (i = 0; languages && languages[i]; i++) {
+    if (g_str_equal (languages[i], "system")) {
+      char **sys_langs = ephy_langs_get_languages ();
+      int j;
+
+      for (j = 0; sys_langs && sys_langs[j]; j++)
+        g_ptr_array_add (langs, g_strdelimit (g_strdup (sys_langs[i]), "-", '_'));
+
+      g_strfreev (sys_langs);
+    } else
+      g_ptr_array_add (langs, g_strdelimit (g_strdup (languages[i]), "-", '_'));
+  }
+
+  g_ptr_array_add (langs, NULL);
+
+  return (char **)g_ptr_array_free (langs, FALSE);
+}
+
 /* Based on Christian Persch's code from gecko backend of epiphany
    (old transform_accept_languages_list() function) */
 static void
@@ -319,6 +345,8 @@ webkit_pref_callback_accept_languages (GSettings *settings,
   GArray *array;
   char **languages;
   int i;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+  WebKitWebContext *web_context = ephy_embed_shell_get_web_context (shell);
 
   languages = g_settings_get_strv (settings, key);
 
@@ -335,8 +363,13 @@ webkit_pref_callback_accept_languages (GSettings *settings,
 
   ephy_langs_sanitise (array);
 
-  webkit_web_context_set_preferred_languages (webkit_web_context_get_default (),
-                                              (const char * const *)array->data);
+  webkit_web_context_set_preferred_languages (web_context, (const char * const *)(void *)array->data);
+
+  if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING)) {
+    char **normalized = normalize_languages ((char **)(void *)array->data);
+    webkit_web_context_set_spell_checking_languages (web_context, (const char * const *)normalized);
+    g_strfreev (normalized);
+  }
 
   g_strfreev (languages);
   g_array_free (array, TRUE);
@@ -370,23 +403,21 @@ webkit_pref_callback_cookie_accept_policy (GSettings *settings,
 {
   WebKitCookieManager *cookie_manager;
   char *value;
+  EphyEmbedShell *shell;
 
   value = g_settings_get_string (settings, key);
   if (!value)
     return;
 
-  cookie_manager = webkit_web_context_get_cookie_manager (webkit_web_context_get_default ());
+  shell = ephy_embed_shell_get_default ();
+  cookie_manager = webkit_web_context_get_cookie_manager (ephy_embed_shell_get_web_context (shell));
   ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, value);
   g_free (value);
 }
 
 static void
-webkit_pref_callback_gnome_fonts (GSettings *ephy_settings,
-                                  char *key,
-                                  gpointer data)
+ephy_embed_prefs_update_font_settings (GSettings *ephy_settings, char *key)
 {
-  GSettings *settings;
-
   if (g_settings_get_boolean (ephy_settings, key)) {
     g_object_set (webkit_settings,
                   "default-font-family", "serif",
@@ -397,48 +428,28 @@ webkit_pref_callback_gnome_fonts (GSettings *ephy_settings,
                   NULL);
   } else {
     /* Sync with Epiphany values */
-    settings = ephy_settings;
-
-    webkit_pref_callback_font_size (settings, EPHY_PREFS_WEB_SERIF_FONT,
+    webkit_pref_callback_font_size (ephy_settings, EPHY_PREFS_WEB_SERIF_FONT,
                                     "default-font-size");
-    webkit_pref_callback_font_size (settings, EPHY_PREFS_WEB_MONOSPACE_FONT,
+    webkit_pref_callback_font_size (ephy_settings, EPHY_PREFS_WEB_MONOSPACE_FONT,
                                     "default-monospace-font-size");
 
-    webkit_pref_callback_font_family (settings, EPHY_PREFS_WEB_SERIF_FONT,
+    webkit_pref_callback_font_family (ephy_settings, EPHY_PREFS_WEB_SERIF_FONT,
                                       "default-font-family");
-    webkit_pref_callback_font_family (settings, EPHY_PREFS_WEB_SANS_SERIF_FONT,
+    webkit_pref_callback_font_family (ephy_settings, EPHY_PREFS_WEB_SANS_SERIF_FONT,
                                       "sans-serif-font-family");
-    webkit_pref_callback_font_family (settings, EPHY_PREFS_WEB_MONOSPACE_FONT,
+    webkit_pref_callback_font_family (ephy_settings, EPHY_PREFS_WEB_MONOSPACE_FONT,
                                       "monospace-font-family");
-    webkit_pref_callback_font_family (settings, EPHY_PREFS_WEB_SERIF_FONT,
+    webkit_pref_callback_font_family (ephy_settings, EPHY_PREFS_WEB_SERIF_FONT,
                                       "serif-font-family");
   }
 }
 
-static char **
-normalize_languages (char **languages)
+static void
+webkit_pref_callback_gnome_fonts (GSettings *ephy_settings,
+                                  char *key,
+                                  gpointer data)
 {
-  int i;
-  GPtrArray *langs;
-
-  langs = g_ptr_array_new ();
-
-  for (i = 0; languages && languages[i]; i++) {
-    if (g_str_equal (languages[i], "system")) {
-      char **sys_langs = ephy_langs_get_languages ();
-      int j;
-
-      for (j = 0; sys_langs && sys_langs[j]; j++)
-        g_ptr_array_add (langs, g_strdelimit (g_strdup (sys_langs[i]), "-", '_'));
-
-      g_strfreev (sys_langs);
-    } else
-      g_ptr_array_add (langs, g_strdelimit (g_strdup (languages[i]), "-", '_'));
-  }
-
-  g_ptr_array_add (langs, NULL);
-
-  return (char **)g_ptr_array_free (langs, FALSE);
+  ephy_embed_prefs_update_font_settings (ephy_settings, key);
 }
 
 static void
@@ -446,30 +457,33 @@ webkit_pref_callback_enable_spell_checking (GSettings *settings,
                                             char *key,
                                             gpointer data)
 {
-  WebKitWebContext *web_context = NULL;
+  WebKitWebContext *web_context;
   gboolean value = FALSE;
-  char **languages = NULL;
-  char *langs = NULL;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
+  web_context = ephy_embed_shell_get_web_context (shell);
   value = g_settings_get_boolean (settings, key);
 
-  if (value) {
-    char **normalized;
-
-    languages = g_settings_get_strv (settings, EPHY_PREFS_WEB_LANGUAGE);
-    normalized = normalize_languages (languages);
-    g_strfreev (languages);
-
-    languages = normalized;
-    langs = g_strjoinv (",", languages);
-  }
-
-  web_context = webkit_web_context_get_default ();
   webkit_web_context_set_spell_checking_enabled (web_context, value);
-  webkit_web_context_set_spell_checking_languages (web_context, (const char* const *)languages);
 
-  g_free (langs);
-  g_strfreev (languages);
+  if (value) {
+    char **languages = g_settings_get_strv (settings, EPHY_PREFS_WEB_LANGUAGE);
+    char **normalized = normalize_languages (languages);
+
+    webkit_web_context_set_spell_checking_languages (web_context, (const char* const *)normalized);
+
+    g_strfreev (languages);
+    g_strfreev (normalized);
+  }
+}
+
+static void
+gtk_settings_xft_dpi_changed_cb (GtkSettings *gtk_settings,
+                                 GParamSpec *pspec,
+                                 gpointer data)
+{
+  GSettings *gsettings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
+  ephy_embed_prefs_update_font_settings (gsettings, EPHY_PREFS_WEB_USE_GNOME_FONTS);
 }
 
 static const PrefData webkit_pref_entries[] =
@@ -531,6 +545,7 @@ static const PrefData webkit_pref_entries[] =
 static gpointer
 ephy_embed_prefs_init (gpointer user_data)
 {
+  GtkSettings *gtk_settings;
   int i;
 
   webkit_settings = webkit_settings_new_with_settings ("enable-developer-extras", TRUE,
@@ -557,6 +572,16 @@ ephy_embed_prefs_init (gpointer user_data)
     g_free (key);
   }
 
+  /* Connect to the "notify::gtk-xft-dpi" signal for GtkSettings, so that
+   * we can update the font size in real time if the screen's resolution
+   * for font handling changes (e.g. enabled "Large Text" a11y mode).
+   */
+  gtk_settings = gtk_settings_get_default ();
+  if (gtk_settings) {
+    g_signal_connect (gtk_settings, "notify::gtk-xft-dpi",
+                      G_CALLBACK (gtk_settings_xft_dpi_changed_cb), NULL);
+  }
+
   g_settings_bind (EPHY_SETTINGS_WEB,
                    EPHY_PREFS_WEB_ENABLE_JAVASCRIPT,
                    webkit_settings, ENABLE_SCRIPTS_SETTING,
