@@ -29,6 +29,7 @@
 #define EPHY_EMBED_UTILS_H
 
 #include "ephy-web-view.h"
+#include <webkit2/webkit2.h>
 
 G_BEGIN_DECLS
 
@@ -42,12 +43,15 @@ gboolean ephy_embed_utils_address_has_web_scheme                (const char *add
 gboolean ephy_embed_utils_address_is_existing_absolute_filename (const char *address);
 gboolean ephy_embed_utils_address_is_valid                      (const char *address);
 char*    ephy_embed_utils_normalize_address                     (const char *address);
+char *   ephy_embed_utils_autosearch_address                    (const char *search_key);
 char *   ephy_embed_utils_normalize_or_autosearch_address       (const char *address);
 gboolean ephy_embed_utils_url_is_empty                          (const char *location);
 gboolean ephy_embed_utils_is_no_show_address                    (const char *address);
 char    *ephy_embed_utils_get_title_from_address                (const char *address);
 gboolean ephy_embed_utils_urls_have_same_origin                 (const char *a_url,
                                                                  const char *b_url);
+char    *ephy_embed_utils_get_js_result_as_string               (WebKitJavascriptResult *js_result);
+double   ephy_embed_utils_get_js_result_as_number               (WebKitJavascriptResult *js_result);
 void     ephy_embed_utils_shutdown                              (void);
 
 G_END_DECLS
