@@ -30,19 +30,8 @@ struct _EphyWebExtensionProxyPrivate
   gchar *name_owner;
   GCancellable *cancellable;
   guint watch_name_id;
-  guint form_auth_save_signal_id;
 };
 
-enum
-{
-  FORM_AUTH_DATA_SAVE_REQUESTED,
-  ALLOW_TLS_CERTIFICATE,
-
-  LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL];
-
 G_DEFINE_TYPE (EphyWebExtensionProxy, ephy_web_extension_proxy, G_TYPE_OBJECT)
 
 static void
@@ -86,46 +75,6 @@ ephy_web_extension_proxy_class_init (EphyWebExtensionProxyClass *klass)
   object_class->finalize = ephy_web_extension_proxy_finalize;
   object_class->dispose = ephy_web_extension_proxy_dispose;
 
-  /**
-   * EphyWebExtensionProxy::form-auth-data-save-requested:
-   * @web_extension: the #EphyWebExtensionProxy
-   * @request_id: the identifier of the request
-   * @page_id: the identifier of the web page
-   * @hostname: the hostname
-   * @username: the username
-   *
-   * Emitted when a web page requests confirmation to save
-   * the form authentication data for the given @hostname and
-   * @username
-   **/
-  signals[FORM_AUTH_DATA_SAVE_REQUESTED] =
-    g_signal_new ("form-auth-data-save-requested",
-                  EPHY_TYPE_WEB_EXTENSION_PROXY,
-                  G_SIGNAL_RUN_FIRST,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_generic,
-                  G_TYPE_NONE, 4,
-                  G_TYPE_UINT,
-                  G_TYPE_UINT64,
-                  G_TYPE_STRING,
-                  G_TYPE_STRING);
-
-  /**
-   * EphyWebExtensionProxy::allow-tls-certificate:
-   * @shell: the #EphyWebExtensionProxy
-   *
-   * Emitted when the web extension requests an exception be
-   * permitted for the invalid TLS certificate on the current page.
-   */
-  signals[ALLOW_TLS_CERTIFICATE] =
-    g_signal_new ("allow-tls-certificate",
-                  EPHY_TYPE_WEB_EXTENSION_PROXY,
-                  G_SIGNAL_RUN_FIRST,
-                  0, NULL, NULL,
-                  g_cclosure_marshal_generic,
-                  G_TYPE_NONE, 1,
-                  G_TYPE_UINT64);
-
   g_type_class_add_private (object_class, sizeof (EphyWebExtensionProxyPrivate));
 }
 
@@ -141,6 +90,8 @@ web_extension_proxy_created_cb (GDBusProxy *proxy,
     g_warning ("Error creating web extension proxy: %s\n", error->message);
     g_error_free (error);
   }
+
+  g_object_unref (web_extension);
 }
 
 static void
@@ -161,7 +112,9 @@ web_extension_appeared_cb (GDBusConnection *connection,
                     EPHY_WEB_EXTENSION_INTERFACE,
                     web_extension->priv->cancellable,
                     (GAsyncReadyCallback)web_extension_proxy_created_cb,
-                    web_extension);
+                    /* Ref here because the web process could crash, triggering
+                     * web_extension_vanished_cb() before this finishes. */
+                    g_object_ref (web_extension));
 }
 
 static void
@@ -213,19 +166,6 @@ ephy_web_extension_proxy_get_name_owner (EphyWebExtensionProxy *web_extension)
   return web_extension->priv->name_owner;
 }
 
-void
-ephy_web_extension_proxy_form_auth_save_requested (EphyWebExtensionProxy *web_extension,
-                                                   guint request_id,
-                                                   guint64 page_id,
-                                                   const char *hostname,
-                                                   const char *username)
-{
-  g_return_if_fail (EPHY_IS_WEB_EXTENSION_PROXY (web_extension));
-
-  g_signal_emit (web_extension, signals[FORM_AUTH_DATA_SAVE_REQUESTED], 0,
-                 request_id, page_id, hostname, username);
-}
-
 void
 ephy_web_extension_proxy_form_auth_data_save_confirmation_response (EphyWebExtensionProxy *web_extension,
                                                                     guint request_id,
@@ -245,15 +185,6 @@ ephy_web_extension_proxy_form_auth_data_save_confirmation_response (EphyWebExten
                      NULL, NULL);
 }
 
-void
-ephy_web_extension_proxy_allow_tls_certificate (EphyWebExtensionProxy *web_extension,
-                                                guint64 page_id)
-{
-  g_return_if_fail (EPHY_IS_WEB_EXTENSION_PROXY (web_extension));
-
-  g_signal_emit (web_extension, signals[ALLOW_TLS_CERTIFICATE], 0, page_id);
-}
-
 static void
 has_modified_forms_cb (GDBusProxy *proxy,
                        GAsyncResult *result,
