@@ -57,6 +57,7 @@ gboolean           ephy_file_helpers_init                   (const char
 const char *       ephy_file                                (const char            *filename);
 const char *       ephy_dot_dir                             (void);
 gboolean           ephy_dot_dir_is_default                  (void);
+char       *       ephy_default_dot_dir                     (void);
 void               ephy_file_helpers_shutdown               (void);
 char	   *          ephy_file_get_downloads_dir              (void);
 char       *       ephy_file_desktop_dir                    (void);
@@ -86,7 +87,7 @@ gboolean           ephy_file_open_uri_in_default_browser    (const char
                                                              GdkScreen             *screen);
 gboolean           ephy_file_browse_to                      (GFile                 *file,
                                                              guint32                user_time);
-gboolean           ephy_file_delete_dir_recursively         (GFile                 *file,
+gboolean           ephy_file_delete_dir_recursively         (const char            *directory,
                                                              GError               **error);
 void               ephy_file_delete_uri                     (const char            *uri);
 gboolean           ephy_file_move_uri                       (const char            *source_uri,
@@ -96,6 +97,7 @@ char       *       ephy_file_create_data_uri_for_filename   (const char
 char       *       ephy_sanitize_filename                   (char                  *filename);
 GAppInfo   *       ephy_file_launcher_get_app_info_for_file (GFile                 *file,
                                                              const char            *mime_type);
+void               ephy_open_incognito_window               (const char *uri);
 
 G_END_DECLS
 
