@@ -62,8 +62,17 @@ ephy_web_dom_utils_has_modified_forms (WebKitDOMDocument *document)
       element = webkit_dom_html_collection_item (elements, j);
 
       if (WEBKIT_DOM_IS_HTML_TEXT_AREA_ELEMENT (element))
-        if (webkit_dom_html_text_area_element_is_edited (WEBKIT_DOM_HTML_TEXT_AREA_ELEMENT (element)))
-          return TRUE;
+        if (webkit_dom_html_text_area_element_is_edited (WEBKIT_DOM_HTML_TEXT_AREA_ELEMENT (element))) {
+          char *text;
+          gboolean has_data;
+
+          text = webkit_dom_html_text_area_element_get_value (WEBKIT_DOM_HTML_TEXT_AREA_ELEMENT (element));
+          has_data = text && *text;
+          g_free (text);
+
+          if (has_data)
+            return TRUE;
+        }
 
       if (WEBKIT_DOM_IS_HTML_INPUT_ELEMENT (element))
         if (webkit_dom_html_input_element_is_edited (WEBKIT_DOM_HTML_INPUT_ELEMENT (element))) {
@@ -375,7 +384,9 @@ ephy_web_dom_utils_find_form_auth_elements (WebKitDOMHTMLFormElement *form,
 
     g_object_get (element, "type", &element_type, "name", &element_name, NULL);
 
-    if (g_strcmp0 (element_type, "text") == 0 || g_strcmp0 (element_type, "email") == 0) {
+    if (g_strcmp0 (element_type, "text") == 0 ||
+        g_strcmp0 (element_type, "email") == 0 ||
+        g_strcmp0 (element_type, "tel") == 0) {
       /* We found more than one inputs of type text; we won't be saving here. */
       if (username_node) {
         g_free (element_type);
@@ -488,3 +499,25 @@ ephy_web_dom_utils_get_absolute_bottom_for_element (WebKitDOMElement *element,
 
   *y += offset_height;
 }
+
+/**
+ * ephy_web_dom_utils_get_selection_as_string:
+ * @selection: the #WebKitDOMDOMSelection.
+ *
+ * Gets the contents of the selection as a string.
+ *
+ * Returns: a newly allocated string with the selection or %NULL.
+ **/
+char *
+ephy_web_dom_utils_get_selection_as_string   (WebKitDOMDOMSelection *selection)
+{
+  char *string;
+  WebKitDOMRange *range = webkit_dom_dom_selection_get_range_at (selection, 0, NULL);
+  if (!range)
+    return NULL;
+
+  string = webkit_dom_range_to_string (range, NULL);
+  g_object_unref (range);
+
+  return string;
+}
