@@ -30,6 +30,8 @@ typedef enum {
   /* WRITE */
   SET_URL_TITLE,
   SET_URL_ZOOM_LEVEL,
+  SET_URL_HIDDEN,
+  SET_URL_THUMBNAIL_TIME,
   ADD_VISIT,
   ADD_VISITS,
   DELETE_URLS,
@@ -48,7 +50,11 @@ typedef enum {
 
 enum {
   VISIT_URL,
+  URLS_VISITED,
   CLEARED,
+  URL_TITLE_CHANGED,
+  URL_DELETED,
+  HOST_DELETED,
   LAST_SIGNAL
 };
 
@@ -74,6 +80,7 @@ static void ephy_history_service_quit                                     (EphyH
 enum {
   PROP_0,
   PROP_HISTORY_FILENAME,
+  PROP_READ_ONLY
 };
 
 #define EPHY_HISTORY_SERVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), EPHY_TYPE_HISTORY_SERVICE, EphyHistoryServicePrivate))
@@ -90,6 +97,9 @@ ephy_history_service_set_property (GObject *object, guint property_id, const GVa
       g_free (self->priv->history_filename);
       self->priv->history_filename = g_strdup (g_value_get_string (value));
       break;
+    case PROP_READ_ONLY:
+      self->priv->read_only = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
       break;
@@ -125,6 +135,38 @@ ephy_history_service_finalize (GObject *self)
   G_OBJECT_CLASS (ephy_history_service_parent_class)->finalize (self);
 }
 
+static void
+ephy_history_service_dispose (GObject *self)
+{
+  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
+
+  if (priv->queue_urls_visited_id) {
+    g_source_remove (priv->queue_urls_visited_id);
+    priv->queue_urls_visited_id = 0;
+  }
+
+  G_OBJECT_CLASS (ephy_history_service_parent_class)->dispose (self);
+}
+
+static gboolean
+emit_urls_visited (EphyHistoryService *self)
+{
+  g_signal_emit (self, signals[URLS_VISITED], 0);
+  self->priv->queue_urls_visited_id = 0;
+
+  return FALSE;
+}
+
+static void
+ephy_history_service_queue_urls_visited (EphyHistoryService *self)
+{
+  if (self->priv->queue_urls_visited_id)
+    return;
+
+  self->priv->queue_urls_visited_id =
+    g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)emit_urls_visited, self, NULL);
+}
+
 static gboolean
 impl_visit_url (EphyHistoryService *self, const char *url, EphyHistoryPageVisitType visit_type)
 {
@@ -137,6 +179,8 @@ impl_visit_url (EphyHistoryService *self, const char *url, EphyHistoryPageVisitT
                                   visit, NULL, NULL, NULL);
   ephy_history_page_visit_free (visit);
 
+  ephy_history_service_queue_urls_visited (self);
+
   return FALSE;
 }
 
@@ -146,6 +190,7 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize = ephy_history_service_finalize;
+  gobject_class->dispose  = ephy_history_service_dispose;
   gobject_class->get_property = ephy_history_service_get_property;
   gobject_class->set_property = ephy_history_service_set_property;
 
@@ -163,6 +208,25 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
                   G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                   EPHY_TYPE_HISTORY_PAGE_VISIT_TYPE);
 
+/**
+ * EphyHistoryService::urls-visited:
+ * @service: the #EphyHistoryService that received the signal
+ *
+ * The ::urls-visited signal is emitted after one or more visits to
+ * URLS have taken place. This signal is intended for use-cases when
+ * precise information of the actual URLS visited is not important and
+ * there is only interest in the fact that there have been changes in
+ * the history. For more precise information, you can use ::visit-url
+ **/
+  signals[URLS_VISITED] =
+    g_signal_new ("urls-visited",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE,
+                  0);
+
   signals[CLEARED] =
     g_signal_new ("cleared",
                   G_OBJECT_CLASS_TYPE (gobject_class),
@@ -172,6 +236,37 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
                   G_TYPE_NONE,
                   0);
 
+  signals[URL_TITLE_CHANGED] =
+    g_signal_new ("url-title-changed",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_generic,
+                  G_TYPE_NONE,
+                  2,
+                  G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
+                  G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
+  signals[URL_DELETED] =
+    g_signal_new ("url-deleted",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__STRING,
+                  G_TYPE_NONE,
+                  1,
+                  G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
+  signals[HOST_DELETED] =
+    g_signal_new ("host-deleted",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__STRING,
+                  G_TYPE_NONE,
+                  1,
+                  G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
   g_object_class_install_property (gobject_class,
                                    PROP_HISTORY_FILENAME,
                                    g_param_spec_string ("history-filename",
@@ -179,6 +274,13 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
                                                         "The filename of the SQLite file holding containing history",
                                                         NULL,
                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+    g_object_class_install_property (gobject_class,
+                                     PROP_READ_ONLY,
+                                     g_param_spec_boolean ("read-only",
+                                                           "Read only mode",
+                                                           "Whether the history service works in read only mode",
+                                                           FALSE,
+                                                           G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
   g_type_class_add_private (gobject_class, sizeof (EphyHistoryServicePrivate));
 }
@@ -193,10 +295,12 @@ ephy_history_service_init (EphyHistoryService *self)
 }
 
 EphyHistoryService *
-ephy_history_service_new (const char *history_filename)
+ephy_history_service_new (const char *history_filename,
+                          gboolean read_only)
 {
   return EPHY_HISTORY_SERVICE (g_object_new (EPHY_TYPE_HISTORY_SERVICE,
                                              "history-filename", history_filename,
+                                             "read-only", read_only,
                                               NULL));
 }
 
@@ -206,6 +310,40 @@ sort_messages (EphyHistoryServiceMessage* a, EphyHistoryServiceMessage* b, gpoin
   return a->type > b->type ? 1 : a->type == b->type ? 0 : -1;
 }
 
+static EphyHistoryServiceMessage *
+ephy_history_service_message_new (EphyHistoryService *service,
+                                  EphyHistoryServiceMessageType type,
+                                  gpointer method_argument,
+                                  GDestroyNotify method_argument_cleanup,
+                                  GCancellable *cancellable,
+                                  EphyHistoryJobCallback callback,
+                                  gpointer user_data)
+{
+  EphyHistoryServiceMessage *message = g_slice_alloc0 (sizeof (EphyHistoryServiceMessage));
+
+  message->service = service;
+  message->type = type;
+  message->method_argument = method_argument;
+  message->method_argument_cleanup = method_argument_cleanup;
+  message->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+  message->callback = callback;
+  message->user_data = user_data;
+
+  return message;
+}
+
+static void
+ephy_history_service_message_free (EphyHistoryServiceMessage *message)
+{
+  if (message->method_argument_cleanup)
+    message->method_argument_cleanup (message->method_argument);
+
+  if (message->cancellable)
+    g_object_unref (message->cancellable);
+
+  g_slice_free1 (sizeof (EphyHistoryServiceMessage), message);
+}
+
 static void
 ephy_history_service_send_message (EphyHistoryService *self, EphyHistoryServiceMessage *message)
 {
@@ -224,14 +362,17 @@ ephy_history_service_commit (EphyHistoryService *self)
   if (NULL == priv->history_database)
     return;
 
+  if (priv->read_only)
+    return;
+
   ephy_sqlite_connection_commit_transaction (priv->history_database, &error);
   if (NULL != error) {
-    g_error ("Could not commit idle history database transaction: %s", error->message);
+    g_warning ("Could not commit idle history database transaction: %s", error->message);
     g_error_free (error);
   }
   ephy_sqlite_connection_begin_transaction (priv->history_database, &error);
   if (NULL != error) {
-    g_error ("Could not start long-running history database transaction: %s", error->message);
+    g_warning ("Could not start long-running history database transaction: %s", error->message);
     g_error_free (error);
   }
 
@@ -251,7 +392,7 @@ ephy_history_service_enable_foreign_keys (EphyHistoryService *self)
                                   "PRAGMA foreign_keys = ON", &error);
 
   if (error) {
-    g_error ("Could not enable foreign keys pragma: %s", error->message);
+    g_warning ("Could not enable foreign keys pragma: %s", error->message);
     g_error_free (error);
   }
 }
@@ -269,23 +410,26 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
   if (error) {
     g_object_unref (priv->history_database);
     priv->history_database = NULL;
-    g_error ("Could not open history database: %s", error->message);
+    g_warning ("Could not open history database at %s: %s", priv->history_filename, error->message);
     g_error_free (error);
     return FALSE;
   }
 
   ephy_history_service_enable_foreign_keys (self);
 
+  if (priv->read_only)
+    return TRUE;
+
   ephy_sqlite_connection_begin_transaction (priv->history_database, &error);
   if (error) {
-    g_error ("Could not begin long running transaction in history database: %s", error->message);
+    g_warning ("Could not begin long running transaction in history database: %s", error->message);
     g_error_free (error);
     return FALSE;
   }
 
-  if ((ephy_history_service_initialize_hosts_table (self) == FALSE) ||
-      (ephy_history_service_initialize_urls_table (self) == FALSE) ||
-      (ephy_history_service_initialize_visits_table (self) == FALSE))
+  if (ephy_history_service_initialize_hosts_table (self) == FALSE ||
+      ephy_history_service_initialize_urls_table (self) == FALSE ||
+      ephy_history_service_initialize_visits_table (self) == FALSE)
     return FALSE;
 
   return TRUE;
@@ -312,10 +456,13 @@ ephy_history_service_clear_all (EphyHistoryService *self)
   if (NULL == priv->history_database)
     return;
 
+  if (priv->read_only)
+    return;
+
   ephy_sqlite_connection_execute (priv->history_database,
                                   "DELETE FROM hosts;", &error);
   if (error) {
-    g_error ("Couldn't clear history database: %s", error->message);
+    g_warning ("Couldn't clear history database: %s", error->message);
     g_error_free(error);
   }
 }
@@ -335,7 +482,8 @@ ephy_history_service_is_scheduled_to_commit (EphyHistoryService *self)
 void
 ephy_history_service_schedule_commit (EphyHistoryService *self)
 {
-  self->priv->scheduled_to_commit = TRUE;
+  if (!self->priv->read_only)
+    self->priv->scheduled_to_commit = TRUE;
 }
 
 static gboolean
@@ -382,57 +530,24 @@ run_history_service_thread (EphyHistoryService *self)
   } while (!ephy_history_service_is_scheduled_to_quit (self));
 
   ephy_history_service_close_database_connections (self);
-  ephy_history_service_execute_quit (self, NULL, NULL);
 
   return NULL;
 }
 
-static EphyHistoryServiceMessage *
-ephy_history_service_message_new (EphyHistoryService *service,
-                                  EphyHistoryServiceMessageType type,
-                                  gpointer method_argument,
-                                  GDestroyNotify method_argument_cleanup,
-                                  GCancellable *cancellable,
-                                  EphyHistoryJobCallback callback,
-                                  gpointer user_data)
-{
-  EphyHistoryServiceMessage *message = g_slice_alloc0 (sizeof (EphyHistoryServiceMessage));
-
-  message->service = service; 
-  message->type = type;
-  message->method_argument = method_argument;
-  message->method_argument_cleanup = method_argument_cleanup;
-  message->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
-  message->callback = callback;
-  message->user_data = user_data;
-
-  return message;
-}
-
-static void
-ephy_history_service_message_free (EphyHistoryServiceMessage *message)
-{
-  if (message->method_argument_cleanup)
-    message->method_argument_cleanup (message->method_argument);
-
-  if (message->cancellable)
-    g_object_unref (message->cancellable);
-
-  g_slice_free1 (sizeof (EphyHistoryServiceMessage), message);
-}
-
 static gboolean
 ephy_history_service_execute_job_callback (gpointer data)
 {
   EphyHistoryServiceMessage *message = (EphyHistoryServiceMessage*) data;
 
-  g_assert (message->callback);
+  g_assert (message->callback || message->type == CLEAR);
 
   if (g_cancellable_is_cancelled (message->cancellable)) {
     ephy_history_service_message_free (message);
     return FALSE;
   }
-  message->callback (message->service, message->success, message->result, message->user_data);
+
+  if (message->callback)
+    message->callback (message->service, message->success, message->result, message->user_data);
 
   if (message->type == CLEAR)
     g_signal_emit (message->service, signals[CLEARED], 0);
@@ -442,6 +557,35 @@ ephy_history_service_execute_job_callback (gpointer data)
   return FALSE;
 }
 
+typedef struct {
+  EphyHistoryService *service;
+  gpointer user_data;
+  GDestroyNotify destroy_func;
+} SignalEmissionContext;
+
+static void
+signal_emission_context_free (SignalEmissionContext *ctx)
+{
+  g_object_unref (ctx->service);
+  if (ctx->destroy_func && ctx->user_data)
+    ctx->destroy_func (ctx->user_data);
+  g_slice_free (SignalEmissionContext, ctx);
+}
+
+static SignalEmissionContext *
+signal_emission_context_new (EphyHistoryService *service,
+                             gpointer user_data,
+                             GDestroyNotify destroy_func)
+{
+  SignalEmissionContext *ctx = g_slice_new0 (SignalEmissionContext);
+
+  ctx->service = g_object_ref (service);
+  ctx->user_data = user_data;
+  ctx->destroy_func = destroy_func;
+
+  return ctx;
+}
+
 static gboolean
 ephy_history_service_execute_add_visit_helper (EphyHistoryService *self, EphyHistoryPageVisit *visit)
 {
@@ -468,7 +612,7 @@ ephy_history_service_execute_add_visit_helper (EphyHistoryService *self, EphyHis
     ephy_history_service_add_url_row (self, visit->url);
 
     if (visit->url->id == -1) {
-      g_error ("Adding visit failed after failed URL addition.");
+      g_warning ("Adding visit failed after failed URL addition.");
       return FALSE;
     }
 
@@ -491,6 +635,9 @@ ephy_history_service_execute_add_visit (EphyHistoryService *self, EphyHistoryPag
   gboolean success;
   g_assert (self->priv->history_thread == g_thread_self ());
 
+  if (self->priv->read_only)
+    return FALSE;
+
   success = ephy_history_service_execute_add_visit_helper (self, visit);
   return success;
 }
@@ -501,6 +648,9 @@ ephy_history_service_execute_add_visits (EphyHistoryService *self, GList *visits
   gboolean success = TRUE;
   g_assert (self->priv->history_thread == g_thread_self ());
 
+  if (self->priv->read_only)
+    return FALSE;
+
   while (visits) {
     success = success && ephy_history_service_execute_add_visit_helper (self, (EphyHistoryPageVisit *) visits->data);
     visits = visits->next;
@@ -522,7 +672,7 @@ ephy_history_service_execute_find_visits (EphyHistoryService *self, EphyHistoryQ
     EphyHistoryPageVisit *visit = (EphyHistoryPageVisit *) current->data;
     if (NULL == ephy_history_service_get_url_row (self, NULL, visit->url)) {
       ephy_history_page_visit_list_free (visits);
-      g_error ("Tried to process an orphaned page visit");
+      g_warning ("Tried to process an orphaned page visit");
       return FALSE;
     }
 
@@ -673,6 +823,16 @@ ephy_history_service_query_hosts (EphyHistoryService *self,
   ephy_history_service_send_message (self, message);
 }
 
+static gboolean
+set_url_title_signal_emit (SignalEmissionContext *ctx)
+{
+  EphyHistoryURL *url = (EphyHistoryURL *)ctx->user_data;
+
+  g_signal_emit (ctx->service, signals[URL_TITLE_CHANGED], 0, url->url, url->title);
+
+  return FALSE;
+}
+
 static gboolean
 ephy_history_service_execute_set_url_title (EphyHistoryService *self,
                                             EphyHistoryURL *url,
@@ -680,15 +840,27 @@ ephy_history_service_execute_set_url_title (EphyHistoryService *self,
 {
   char *title = g_strdup (url->title);
 
+  if (self->priv->read_only)
+    return FALSE;
+
   if (NULL == ephy_history_service_get_url_row (self, NULL, url)) {
     /* The URL is not yet in the database, so we can't update it.. */
     g_free (title);
     return FALSE;
   } else {
+    SignalEmissionContext *ctx;
+
     g_free (url->title);
     url->title = title;
     ephy_history_service_update_url_row (self, url);
     ephy_history_service_schedule_commit (self);
+
+    ctx = signal_emission_context_new (self,
+                                       ephy_history_url_copy (url),
+                                       (GDestroyNotify)ephy_history_url_free);
+    g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                     (GSourceFunc)set_url_title_signal_emit,
+                     ctx, (GDestroyNotify)signal_emission_context_free);
     return TRUE;
   }
 }
@@ -706,6 +878,8 @@ ephy_history_service_set_url_title (EphyHistoryService *self,
 
   g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
   g_return_if_fail (orig_url != NULL);
+  g_return_if_fail (title != NULL);
+  g_return_if_fail (*title != '\0');
 
   url = ephy_history_url_new (orig_url, title, 0, 0, 0);
   message = ephy_history_service_message_new (self, SET_URL_TITLE,
@@ -723,6 +897,9 @@ ephy_history_service_execute_set_url_zoom_level (EphyHistoryService *self,
   double zoom_level;
   EphyHistoryHost *host;
 
+  if (self->priv->read_only)
+    return FALSE;
+
   g_variant_get (variant, "(sd)", &url_string, &zoom_level);
 
   host = ephy_history_service_get_host_row_from_url (self, url_string);
@@ -759,6 +936,96 @@ ephy_history_service_set_url_zoom_level (EphyHistoryService *self,
   ephy_history_service_send_message (self, message);
 }
 
+static gboolean
+ephy_history_service_execute_set_url_hidden (EphyHistoryService *self,
+                                             EphyHistoryURL *url,
+                                             gpointer *result)
+{
+  gboolean hidden;
+
+  if (self->priv->read_only)
+    return FALSE;
+
+  hidden = url->hidden;
+
+  if (NULL == ephy_history_service_get_url_row (self, NULL, url)) {
+    /* The URL is not yet in the database, so we can't update it.. */
+    return FALSE;
+  } else {
+    url->hidden = hidden;
+    ephy_history_service_update_url_row (self, url);
+    ephy_history_service_schedule_commit (self);
+    return TRUE;
+  }
+}
+
+void
+ephy_history_service_set_url_hidden (EphyHistoryService *self,
+                                     const char *orig_url,
+                                     gboolean hidden,
+                                     GCancellable *cancellable,
+                                     EphyHistoryJobCallback callback,
+                                     gpointer user_data)
+{
+  EphyHistoryServiceMessage *message;
+  EphyHistoryURL *url;
+
+  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_return_if_fail (orig_url != NULL);
+
+  url = ephy_history_url_new (orig_url, NULL, 0, 0, 0);
+  url->hidden = hidden;
+
+  message = ephy_history_service_message_new (self, SET_URL_HIDDEN,
+                                              url, (GDestroyNotify)ephy_history_url_free,
+                                              cancellable, callback, user_data);
+  ephy_history_service_send_message (self, message);
+}
+
+static gboolean
+ephy_history_service_execute_set_url_thumbnail_time (EphyHistoryService *self,
+                                                     EphyHistoryURL *url,
+                                                     gpointer *result)
+{
+  int thumbnail_time;
+
+  if (self->priv->read_only)
+    return FALSE;
+
+  thumbnail_time = url->thumbnail_time;
+
+  if (NULL == ephy_history_service_get_url_row (self, NULL, url))
+    return FALSE;
+  else {
+    url->thumbnail_time = thumbnail_time;
+    ephy_history_service_update_url_row (self, url);
+    ephy_history_service_schedule_commit (self);
+    return TRUE;
+  }
+}
+
+void
+ephy_history_service_set_url_thumbnail_time (EphyHistoryService *self,
+                                             const char *orig_url,
+                                             int thumbnail_time,
+                                             GCancellable *cancellable,
+                                             EphyHistoryJobCallback callback,
+                                             gpointer user_data)
+{
+  EphyHistoryURL *url;
+  EphyHistoryServiceMessage *message;
+
+  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_return_if_fail (orig_url != NULL);
+
+  url = ephy_history_url_new (orig_url, NULL, 0, 0, 0);
+  url->thumbnail_time = thumbnail_time;
+  message = ephy_history_service_message_new (self, SET_URL_THUMBNAIL_TIME,
+                                              url, (GDestroyNotify) ephy_history_url_free,
+                                              cancellable, callback, user_data);
+  ephy_history_service_send_message (self, message);
+}
+
 static gboolean
 ephy_history_service_execute_get_url (EphyHistoryService *self,
                                       const gchar *orig_url,
@@ -824,6 +1091,16 @@ ephy_history_service_get_host_for_url (EphyHistoryService *self,
   ephy_history_service_send_message (self, message);
 }
 
+static gboolean
+delete_urls_signal_emit (SignalEmissionContext *ctx)
+{
+  char *url = (char *)ctx->user_data;
+
+  g_signal_emit (ctx->service, signals[URL_DELETED], 0, url);
+
+  return FALSE;
+}
+
 static gboolean
 ephy_history_service_execute_delete_urls (EphyHistoryService *self,
                                           GList *urls,
@@ -831,10 +1108,21 @@ ephy_history_service_execute_delete_urls (EphyHistoryService *self,
 {
   GList *l;
   EphyHistoryURL *url;
+  SignalEmissionContext *ctx;
+
+  if (self->priv->read_only)
+    return FALSE;
 
   for (l = urls; l != NULL; l = l->next) {
     url = l->data;
     ephy_history_service_delete_url (self, url);
+
+    ctx = signal_emission_context_new (self, g_strdup (url->url),
+                                       (GDestroyNotify) g_free);
+    g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                     (GSourceFunc)delete_urls_signal_emit,
+                     ctx,
+                     (GDestroyNotify)signal_emission_context_free);
   }
 
   ephy_history_service_delete_orphan_hosts (self);
@@ -843,15 +1131,37 @@ ephy_history_service_execute_delete_urls (EphyHistoryService *self,
   return TRUE;
 }
 
+static gboolean
+delete_host_signal_emit (SignalEmissionContext *ctx)
+{
+  char *host = (char *)ctx->user_data;
+
+  g_signal_emit (ctx->service, signals[HOST_DELETED], 0, host);
+
+  return FALSE;
+}
+
 static gboolean
 ephy_history_service_execute_delete_host (EphyHistoryService *self,
                                           EphyHistoryHost *host,
                                           EphyHistoryJobCallback callback,
                                           gpointer user_data)
 {
+  SignalEmissionContext *ctx;
+
+  if (self->priv->read_only)
+    return FALSE;
+
   ephy_history_service_delete_host_row (self, host);
   ephy_history_service_schedule_commit (self);
 
+  ctx = signal_emission_context_new (self, g_strdup (host->url),
+                                     (GDestroyNotify)g_free);
+  g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                   (GSourceFunc) delete_host_signal_emit,
+                   ctx,
+                   (GDestroyNotify) signal_emission_context_free);
+
   return TRUE;
 }
 
@@ -860,6 +1170,8 @@ ephy_history_service_execute_clear (EphyHistoryService *self,
                                     gpointer pointer,
                                     gpointer *result)
 {
+  if (self->priv->read_only)
+    return FALSE;
 
   ephy_history_service_clear_all (self);
   ephy_history_service_schedule_commit (self);
@@ -930,6 +1242,8 @@ ephy_history_service_quit (EphyHistoryService *self,
 static EphyHistoryServiceMethod methods[] = {
   (EphyHistoryServiceMethod)ephy_history_service_execute_set_url_title,
   (EphyHistoryServiceMethod)ephy_history_service_execute_set_url_zoom_level,
+  (EphyHistoryServiceMethod)ephy_history_service_execute_set_url_hidden,
+  (EphyHistoryServiceMethod)ephy_history_service_execute_set_url_thumbnail_time,
   (EphyHistoryServiceMethod)ephy_history_service_execute_add_visit,
   (EphyHistoryServiceMethod)ephy_history_service_execute_add_visits,
   (EphyHistoryServiceMethod)ephy_history_service_execute_delete_urls,
@@ -966,9 +1280,12 @@ ephy_history_service_process_message (EphyHistoryService *self,
 
   method = methods[message->type];
   message->result = NULL;
-  message->success = method (message->service, message->method_argument, &message->result);
+  if (message->service->priv->history_database)
+    message->success = method (message->service, message->method_argument, &message->result);
+  else
+    message->success = FALSE;
 
-  if (message->callback)
+  if (message->callback || message->type == CLEAR)
     g_idle_add ((GSourceFunc)ephy_history_service_execute_job_callback, message);
   else
     ephy_history_service_message_free (message);
@@ -983,6 +1300,7 @@ ephy_history_service_find_urls (EphyHistoryService *self,
                                 gint64 from, gint64 to,
                                 guint limit, gint host,
                                 GList *substring_list,
+                                EphyHistorySortType sort_type,
                                 GCancellable *cancellable,
                                 EphyHistoryJobCallback callback,
                                 gpointer user_data)
@@ -995,7 +1313,7 @@ ephy_history_service_find_urls (EphyHistoryService *self,
   query->from = from;
   query->to = to;
   query->substring_list = substring_list;
-  query->sort_type = EPHY_HISTORY_SORT_MV;
+  query->sort_type = sort_type;
   query->host = host;
 
   if (limit != 0)
