@@ -48,10 +48,14 @@ typedef enum {
 
 typedef enum {
   EPHY_HISTORY_SORT_NONE = 0,
-  EPHY_HISTORY_SORT_MRV, /* Most recently visited first. */
-  EPHY_HISTORY_SORT_LRV, /* Least recently visited first. */
-  EPHY_HISTORY_SORT_MV,  /* Most visited first. */
-  EPHY_HISTORY_SORT_LV   /* Least visited first. */
+  EPHY_HISTORY_SORT_MOST_RECENTLY_VISITED,
+  EPHY_HISTORY_SORT_LEAST_RECENTLY_VISITED,
+  EPHY_HISTORY_SORT_MOST_VISITED,
+  EPHY_HISTORY_SORT_LEAST_VISITED,
+  EPHY_HISTORY_SORT_TITLE_ASCENDING,
+  EPHY_HISTORY_SORT_TITLE_DESCENDING,
+  EPHY_HISTORY_SORT_URL_ASCENDING,
+  EPHY_HISTORY_SORT_URL_DESCENDING
 } EphyHistorySortType;
 
 typedef struct
@@ -91,6 +95,7 @@ typedef struct _EphyHistoryQuery
   guint limit;
   GList* substring_list;
   gboolean ignore_hidden;
+  gboolean ignore_local;
   gint host;
   EphyHistorySortType sort_type;
 } EphyHistoryQuery;
