@@ -37,6 +37,7 @@
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-initial-state.h"
+#include "ephy-uri-helpers.h"
 #include "ephy-topic-action.h"
 #include "ephy-window.h"
 #include "popup-commands.h"
@@ -72,7 +73,7 @@ static const struct
 export_formats [] = 
 /* Don't add or reoder those entries without updating export_dialog_response_cb too! */
 {
-	{ N_("Epiphany (RDF)"), "rdf" },
+	{ N_("Web (RDF)"), "rdf" },
 	{ N_("Mozilla (HTML)"), "html" }
 };
 
@@ -294,6 +295,17 @@ get_target_window (EphyBookmarksEditor *editor)
 	}
 }
 
+static void
+load_bookmark_in_tab (EphyNode *node,
+                      EphyEmbed *embed)
+{
+        const char *location;
+
+        location = ephy_node_get_property_string (node,
+                                                  EPHY_NODE_BMK_PROP_LOCATION);
+        ephy_web_view_load_url (ephy_embed_get_web_view (embed), location);
+}
+
 static void
 cmd_open_bookmarks_in_tabs (GtkAction *action,
 			    EphyBookmarksEditor *editor)
@@ -309,15 +321,12 @@ cmd_open_bookmarks_in_tabs (GtkAction *action,
 	{
 		EphyNode *node = l->data;
 		EphyEmbed *new_embed;
-		const char *location;
-
-		location = ephy_node_get_property_string (node,
-						EPHY_NODE_BMK_PROP_LOCATION);
 
 		new_embed = ephy_shell_new_tab (ephy_shell_get_default (),
-						window, NULL, location,
-				 	        EPHY_NEW_TAB_OPEN_PAGE |
-					        EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+						window, NULL,
+				 	        0);
+                load_bookmark_in_tab (node, new_embed);
+
 		/* if there was no target window, a new one was opened. Get it
 		 * from the new tab so we open the remaining links in the
 		 * same window. See bug 138343.
@@ -340,21 +349,17 @@ cmd_open_bookmarks_in_browser (GtkAction *action,
 	GList *selection;
 	GList *l;
 
-	window = EPHY_WINDOW (get_target_window (editor));
+	window = ephy_window_new ();
 	selection = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->bm_view));
 
 	for (l = selection; l; l = l->next)
 	{
 		EphyNode *node = l->data;
-		const char *location;
+                EphyEmbed *embed;
 
-		location = ephy_node_get_property_string (node,
-						EPHY_NODE_BMK_PROP_LOCATION);
-
-		ephy_shell_new_tab (ephy_shell_get_default (),
-				    window, NULL, location,
-				    EPHY_NEW_TAB_OPEN_PAGE |
-				    EPHY_NEW_TAB_IN_NEW_WINDOW);
+                embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                                            window, NULL, 0);
+                load_bookmark_in_tab (node, embed);
 	}
 
 	g_list_free (selection);
@@ -397,7 +402,7 @@ cmd_delete (GtkAction *action,
 	}
 	else if (ephy_node_view_is_target (EPHY_NODE_VIEW (editor->priv->key_view)))
 	{
-		EphyNodePriority priority;
+		int priority;
 		GList *selected;
 		EphyNode *node;
 
@@ -626,7 +631,7 @@ import_dialog_response_cb (GtkDialog *dialog,
 				 "application/x-xbel", NULL);
 
 			ephy_file_chooser_add_mime_filter
-				(dialog, _("Epiphany bookmarks"),
+				(dialog, _("Web bookmarks"),
 				 "text/rdf", "application/rdf+xml", NULL);
 
 			filter = ephy_file_chooser_add_pattern_filter (dialog,
@@ -815,7 +820,7 @@ cmd_bookmarks_import (GtkAction *action,
 					      _("_Cancel"),
 					      GTK_RESPONSE_CANCEL,
 					      _("I_mport"),
-					      _("_OK"),
+					      GTK_RESPONSE_OK,
 					      NULL);
 	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
@@ -1017,6 +1022,7 @@ ephy_bookmarks_editor_class_init (EphyBookmarksEditorClass *klass)
 static void
 ephy_bookmarks_editor_finalize (GObject *object)
 {
+	/* FIXME: This should all be done in dispose, not finalize, but got to make sure that's safe. */
 	EphyBookmarksEditor *editor = EPHY_BOOKMARKS_EDITOR (object);
 
 	g_object_unref (G_OBJECT (editor->priv->bookmarks_filter));
@@ -1040,15 +1046,11 @@ ephy_bookmarks_editor_node_activated_cb (GtkWidget *view,
 					 EphyNode *node,
 					 EphyBookmarksEditor *editor)
 {
-	const char *location;
+        EphyEmbed *embed;
 
-	location = ephy_node_get_property_string
-		(node, EPHY_NODE_BMK_PROP_LOCATION);
-	g_return_if_fail (location != NULL);
-
-	ephy_shell_new_tab (ephy_shell_get_default (),
-			    NULL, NULL, location,
-			    EPHY_NEW_TAB_OPEN_PAGE);
+	embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                                    NULL, NULL, 0);
+        load_bookmark_in_tab (node, embed);
 }
 
 static void
@@ -1057,18 +1059,13 @@ ephy_bookmarks_editor_node_middle_clicked_cb (GtkWidget *view,
 					      EphyBookmarksEditor *editor)
 {
 	EphyWindow *window;
-	const char *location;
+        EphyEmbed *embed;
 
 	window = EPHY_WINDOW (get_target_window (editor));
 
-	location = ephy_node_get_property_string
-		(node, EPHY_NODE_BMK_PROP_LOCATION);
-	g_return_if_fail (location != NULL);
-
-	ephy_shell_new_tab (ephy_shell_get_default (),
-			    window, NULL, location,
-			    EPHY_NEW_TAB_OPEN_PAGE |
-			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+	embed = ephy_shell_new_tab (ephy_shell_get_default (),
+                                    window, NULL, 0);
+        load_bookmark_in_tab (node, embed);
 }
 
 static void
@@ -1082,6 +1079,7 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	gboolean key_normal = FALSE;
 	gboolean cut, copy, paste, select_all;
 	gboolean mutable = TRUE;
+        gboolean fullscreen_lockdown;
 
 	GtkActionGroup *action_group;
 	GtkAction *action;
@@ -1129,7 +1127,7 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	if (key_focus && selected)
 	{
 		EphyNode *node = selected->data;
-		EphyNodePriority priority;
+		int priority;
 		char name[EPHY_TOPIC_ACTION_NAME_BUFFER_SIZE];
 
 		priority = ephy_node_get_property_int
@@ -1171,7 +1169,9 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 		copy_label = _("_Copy");
 	}
 
-	open_in_window = (bmk_focus && bmk_selection);
+        fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+                                                      EPHY_PREFS_LOCKDOWN_FULLSCREEN);
+	open_in_window = (bmk_focus && bmk_selection && !fullscreen_lockdown);
 	open_in_tab = (bmk_focus && bmk_selection);
 	rename = (bmk_focus && single_bmk_selected && mutable) ||
 		 (key_selection && key_focus && key_normal);
@@ -1448,89 +1448,14 @@ node_dropped_cb (EphyNodeView *view,
 	}
 }
 
-#ifdef HAVE_WEBKIT2
 /* TODO: Favicons */
-#else
-static gboolean
-webkit_favicon_database_has_favicon (WebKitFaviconDatabase *database, const char *page_uri)
-{
-    gboolean result;
-    char *uri;
-
-    uri = webkit_favicon_database_get_favicon_uri (database, page_uri);
-    result = (uri != NULL);
-    g_free (uri);
-
-    return result;
-}
-
-static void
-icon_loaded_cb (WebKitFaviconDatabase *database, GAsyncResult *result, GtkTreeRowReference *reference)
-{
-    GdkPixbuf *favicon = webkit_favicon_database_get_favicon_pixbuf_finish (database, result, NULL);
-
-    if (favicon && gtk_tree_row_reference_valid (reference)) {
-	    GtkTreeModel *model = gtk_tree_row_reference_get_model (reference);
-	    GtkTreePath *path = gtk_tree_row_reference_get_path (reference);
-	    GtkTreeIter iter;
-
-	    /* Force repaint. */
-	    if (gtk_tree_model_get_iter (model, &iter, path))
-		    gtk_tree_model_row_changed (model, path, &iter);
-
-	    gtk_tree_path_free (path);
-    }
-
-    gtk_tree_row_reference_free (reference);
-    if (favicon)
-	    g_object_unref (favicon);
-}
-#endif
 
 static void
 provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 {
 	GdkPixbuf *favicon = NULL;
-#ifndef HAVE_WEBKIT2
-	const char *page_location;
 
-	page_location = ephy_node_get_property_string
-		(node, EPHY_NODE_BMK_PROP_LOCATION);
-
-	LOG ("Get favicon for %s", page_location ? page_location : "None");
-#endif
-
-#ifdef HAVE_WEBKIT2
         /* TODO: Favicons */
-#else
-	if (page_location)
-        {
-		WebKitFaviconDatabase *database = webkit_get_favicon_database ();
-
-		/* Try with the sync version first as this method will be frequently called. */
-                favicon = webkit_favicon_database_try_get_favicon_pixbuf (database, page_location,
-									  FAVICON_SIZE, FAVICON_SIZE);
-
-		if (!favicon && webkit_favicon_database_has_favicon (database, page_location)) {
-			GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (user_data));
-			GtkTreeIter iter;
-
-			if (ephy_node_view_get_iter_for_node (EPHY_NODE_VIEW (user_data), &iter, node)) {
-				GtkTreeRowReference *reference;
-				GtkTreePath *path;
-
-				path = gtk_tree_model_get_path (model, &iter);
-				reference = gtk_tree_row_reference_new (model, path);
-				gtk_tree_path_free (path);
-
-				webkit_favicon_database_get_favicon_pixbuf (database, page_location,
-									    FAVICON_SIZE, FAVICON_SIZE, NULL,
-									    (GAsyncReadyCallback) icon_loaded_cb,
-									    reference);
-			}
-		}
-        }
-#endif
 	g_value_init (value, GDK_TYPE_PIXBUF);
 	g_value_take_object (value, favicon);
 }
@@ -1554,6 +1479,16 @@ provide_keyword_uri (EphyNode *node, GValue *value, gpointer data)
 	g_free (uri);
 }
 
+static void
+unescape_bookmark_uri (EphyNode *node,
+		       GValue *value,
+		       gpointer user_data)
+{
+	const char *url = g_value_get_string (value);
+	g_value_take_string (value, ephy_uri_safe_unescape (url));
+}
+
+
 static void
 ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 {
@@ -1645,7 +1580,7 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 			  G_CALLBACK (view_selection_changed_cb),
 			  editor);
 	ephy_node_view_set_priority (EPHY_NODE_VIEW (key_view),
-				     EPHY_NODE_KEYWORD_PROP_PRIORITY);
+				     (EphyNodeViewPriority)EPHY_NODE_KEYWORD_PROP_PRIORITY);
 	ephy_node_view_set_sort (EPHY_NODE_VIEW (key_view), G_TYPE_STRING,
 				 EPHY_NODE_KEYWORD_PROP_NAME, GTK_SORT_ASCENDING);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), key_view);
@@ -1705,11 +1640,12 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	gtk_tree_view_column_set_min_width (editor->priv->title_col, 300);
 	gtk_tree_view_column_set_resizable (editor->priv->title_col, TRUE);
 
-	url_col_id = ephy_node_view_add_column
+	url_col_id = ephy_node_view_add_column_full
 				  (EPHY_NODE_VIEW (bm_view), _("Address"),
 				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_LOCATION,
 				   EPHY_NODE_VIEW_SORTABLE |
 				   EPHY_NODE_VIEW_ELLIPSIZED,
+				   unescape_bookmark_uri, NULL,
 				   NULL, &(editor->priv->address_col));
 	gtk_tree_view_column_set_min_width (editor->priv->address_col, 300);
 	gtk_tree_view_column_set_resizable (editor->priv->address_col, TRUE);
