@@ -23,20 +23,33 @@
 
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
+#include "ephy-favicon-helpers.h"
 #include "ephy-history-service.h"
 #include "ephy-shell.h"
+#include "ephy-uri-helpers.h"
 
 #include <string.h>
 
+enum {
+  PROP_0,
+  PROP_HISTORY_SERVICE,
+  PROP_BOOKMARKS,
+  PROP_USE_MARKUP
+};
+
 G_DEFINE_TYPE (EphyCompletionModel, ephy_completion_model, GTK_TYPE_LIST_STORE)
 
 #define EPHY_COMPLETION_MODEL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_COMPLETION_MODEL, EphyCompletionModelPrivate))
 
 struct _EphyCompletionModelPrivate {
   EphyHistoryService *history_service;
+  GCancellable *cancellable;
 
   EphyNode *bookmarks;
+  EphyNode *smart_bookmarks;
   GSList *search_terms;
+
+  gboolean use_markup;
 };
 
 static void
@@ -62,6 +75,31 @@ free_search_terms (GSList *search_terms)
   g_slist_free (search_terms);
 }
 
+static void
+ephy_completion_model_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
+{
+  EphyCompletionModel *self = EPHY_COMPLETION_MODEL (object);
+
+  switch (property_id) {
+  case PROP_HISTORY_SERVICE:
+    self->priv->history_service = EPHY_HISTORY_SERVICE (g_value_get_pointer (value));
+    break;
+  case PROP_BOOKMARKS: {
+    EphyBookmarks *bookmarks = EPHY_BOOKMARKS (g_value_get_pointer (value));
+
+    self->priv->bookmarks = ephy_bookmarks_get_bookmarks (bookmarks);
+    self->priv->smart_bookmarks = ephy_bookmarks_get_smart_bookmarks (bookmarks);
+    }
+    break;
+  case PROP_USE_MARKUP:
+    self->priv->use_markup = g_value_get_boolean (value);
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
+    break;
+  }
+}
+
 static void
 ephy_completion_model_finalize (GObject *object)
 {
@@ -72,6 +110,11 @@ ephy_completion_model_finalize (GObject *object)
     priv->search_terms = NULL;
   }
 
+  if (priv->cancellable) {
+    g_cancellable_cancel (priv->cancellable);
+    g_clear_object (&priv->cancellable);
+  }
+
   G_OBJECT_CLASS (ephy_completion_model_parent_class)->finalize (object);
 }
 
@@ -80,24 +123,38 @@ ephy_completion_model_class_init (EphyCompletionModelClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->set_property = ephy_completion_model_set_property;
   object_class->constructed = ephy_completion_model_constructed;
   object_class->finalize = ephy_completion_model_finalize;
 
+  g_object_class_install_property (object_class,
+                                   PROP_HISTORY_SERVICE,
+                                   g_param_spec_pointer ("history-service",
+                                                         "History Service",
+                                                         "The history service",
+                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_BOOKMARKS,
+                                   g_param_spec_pointer ("bookmarks",
+                                                         "Bookmarks",
+                                                         "The bookmarks",
+                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+                                   PROP_USE_MARKUP,
+                                   g_param_spec_boolean ("use-markup",
+                                                         "Whether we should be using markup",
+                                                         "Whether we should be using markup",
+                                                         TRUE,
+                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
   g_type_class_add_private (object_class, sizeof (EphyCompletionModelPrivate));
 }
 
 static void
 ephy_completion_model_init (EphyCompletionModel *model)
 {
-  EphyCompletionModelPrivate *priv;
-  EphyBookmarks *bookmarks_service;
-
-  model->priv = priv = EPHY_COMPLETION_MODEL_GET_PRIVATE (model);
-
-  priv->history_service = EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (embed_shell));
-
-  bookmarks_service = ephy_shell_get_bookmarks (ephy_shell);
-  priv->bookmarks = ephy_bookmarks_get_bookmarks (bookmarks_service);
+  model->priv = EPHY_COMPLETION_MODEL_GET_PRIVATE (model);
 }
 
 static gboolean
@@ -165,14 +222,20 @@ typedef struct {
   GtkTreeRowReference *row_reference;
 } IconLoadData;
 
-
 static void
 icon_loaded_cb (GObject *source, GAsyncResult *result, gpointer user_data)
 {
   GtkTreeIter iter;
   GtkTreePath *path;
   IconLoadData *data = (IconLoadData *) user_data;
-  GdkPixbuf *favicon = webkit_favicon_database_get_favicon_pixbuf_finish (webkit_get_favicon_database (), result, NULL);
+  WebKitFaviconDatabase *database = WEBKIT_FAVICON_DATABASE (source);
+  GdkPixbuf *favicon = NULL;
+  cairo_surface_t *icon_surface = webkit_favicon_database_get_favicon_finish (database, result, NULL);
+
+  if (icon_surface) {
+    favicon = ephy_pixbuf_get_from_surface_scaled (icon_surface, FAVICON_SIZE, FAVICON_SIZE);
+    cairo_surface_destroy (icon_surface);
+  }
 
   if (favicon) {
     /* The completion model might have changed its contents */
@@ -190,33 +253,51 @@ icon_loaded_cb (GObject *source, GAsyncResult *result, gpointer user_data)
   g_slice_free (IconLoadData, data);
 }
 
+static gchar *
+get_row_text (const gchar *url, const gchar *title, const gchar *subtitle_color)
+{
+  gchar *unescaped_url;
+  gchar *text;
+
+  if (!url)
+    return g_markup_escape_text (title, -1);
+
+  unescaped_url = ephy_uri_safe_unescape (url);
+  if (g_strcmp0 (url, title) == 0)
+    text = g_markup_escape_text (unescaped_url, -1);
+  else
+    text = g_markup_printf_escaped ("%s\n<span font-size=\"small\" color=\"%s\">%s</span>", title, subtitle_color, unescaped_url);
+  g_free (unescaped_url);
+
+  return text;
+}
+
 static void
-set_row_in_model (EphyCompletionModel *model, int position, PotentialRow *row)
+set_row_in_model (EphyCompletionModel *model, int position, PotentialRow *row, const gchar *subtitle_color)
 {
   GtkTreeIter iter;
-  GdkPixbuf *favicon;
   GtkTreePath *path;
   IconLoadData *data;
-  WebKitFaviconDatabase* database = webkit_get_favicon_database ();
+  WebKitFaviconDatabase* database;
+  gchar *text;
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
+
+  database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (shell));
+
+  if (model->priv->use_markup)
+    text = get_row_text (row->location, row->title, subtitle_color);
+  else
+    text = g_strdup (row->title);
 
   gtk_list_store_insert_with_values (GTK_LIST_STORE (model), &iter, position,
-                                     EPHY_COMPLETION_TEXT_COL, row->title ? row->title : "",
+                                     EPHY_COMPLETION_TEXT_COL, text ? text : "",
                                      EPHY_COMPLETION_URL_COL, row->location,
                                      EPHY_COMPLETION_ACTION_COL, row->location,
                                      EPHY_COMPLETION_KEYWORDS_COL, row->keywords ? row->keywords : "",
                                      EPHY_COMPLETION_EXTRA_COL, row->is_bookmark,
                                      EPHY_COMPLETION_RELEVANCE_COL, row->relevance,
                                      -1);
-
-  /* We try first with the try_get_favicon_pixbuf() because if the icon
-     is in the DB it's faster than the async version. */
-  favicon = webkit_favicon_database_try_get_favicon_pixbuf (database, row->location,
-                                                            FAVICON_SIZE, FAVICON_SIZE);
-  if (favicon) {
-    gtk_list_store_set (GTK_LIST_STORE (model), &iter, EPHY_COMPLETION_FAVICON_COL, favicon, -1);
-    g_object_unref (favicon);
-    return;
-  }
+  g_free (text);
 
   data = g_slice_new (IconLoadData);
   data->model = GTK_LIST_STORE (g_object_ref(model));
@@ -224,9 +305,33 @@ set_row_in_model (EphyCompletionModel *model, int position, PotentialRow *row)
   data->row_reference = gtk_tree_row_reference_new (GTK_TREE_MODEL (model), path);
   gtk_tree_path_free (path);
 
-  webkit_favicon_database_get_favicon_pixbuf (database, row->location,
-                                              FAVICON_SIZE, FAVICON_SIZE, NULL,
-                                              icon_loaded_cb, data);
+  webkit_favicon_database_get_favicon (database, row->location,
+                                       NULL, icon_loaded_cb, data);
+}
+
+static gchar *
+get_text_column_subtitle_color (void)
+{
+  GtkWidgetPath *path;
+  GtkStyleContext *style_context;
+  GdkRGBA rgba;
+
+  path = gtk_widget_path_new ();
+  gtk_widget_path_prepend_type (path, GTK_TYPE_ENTRY);
+  gtk_widget_path_iter_add_class (path, 0, GTK_STYLE_CLASS_ENTRY);
+
+  style_context = gtk_style_context_new ();
+  gtk_style_context_set_path (style_context, path);
+  gtk_widget_path_free (path);
+
+  gtk_style_context_add_class (style_context, GTK_STYLE_CLASS_ENTRY);
+  gtk_style_context_get_color (style_context, GTK_STATE_FLAG_INSENSITIVE, &rgba);
+  g_object_unref (style_context);
+
+  return g_strdup_printf ("#%04X%04X%04X",
+                          (guint)(rgba.red * (gdouble)65535),
+                          (guint)(rgba.green * (gdouble)65535),
+                          (guint)(rgba.blue * (gdouble)65535));
 }
 
 static void
@@ -235,15 +340,24 @@ replace_rows_in_model (EphyCompletionModel *model, GSList *new_rows)
   /* This is by far the simplest way of doing, and yet it gives
    * basically the same result than the other methods... */
   int i;
+  gchar *subtitle_color = NULL;
 
   gtk_list_store_clear (GTK_LIST_STORE (model));
 
+  if (!new_rows)
+    return;
+
+  if (model->priv->use_markup)
+    subtitle_color = get_text_column_subtitle_color ();
+
   for (i = 0; new_rows != NULL; i++) {
     PotentialRow *row = (PotentialRow*)new_rows->data;
-    
-    set_row_in_model (model, i, row);
+
+    set_row_in_model (model, i, row, subtitle_color);
     new_rows = new_rows->next;
   }
+
+  g_free (subtitle_color);
 }
 
 static gboolean
@@ -383,14 +497,17 @@ query_completed_cb (EphyHistoryService *service,
   for (i = 0; i < children->len; i++) {
     EphyNode *kid;
     const char *keywords, *location, *title;
+    gboolean is_smart;
 
     kid = g_ptr_array_index (children, i);
     location = ephy_node_get_property_string (kid, EPHY_NODE_BMK_PROP_LOCATION);
     title = ephy_node_get_property_string (kid, EPHY_NODE_BMK_PROP_TITLE);
     keywords = ephy_node_get_property_string (kid, EPHY_NODE_BMK_PROP_KEYWORDS);
+    is_smart = ephy_node_has_child (priv->smart_bookmarks, kid);
 
-    if (should_add_bookmark_to_model (model, user_data->search_string,
-                                      title, location, keywords))
+    /* Smart bookmarks are already added to the completion menu as completion actions */
+    if (!is_smart && should_add_bookmark_to_model (model, user_data->search_string,
+                                                   title, location, keywords))
       list = add_to_potential_rows (list, title, location, keywords, 0, TRUE, FALSE);
   }
 
@@ -416,8 +533,9 @@ query_completed_cb (EphyHistoryService *service,
 
   g_free (user_data->search_string);
   g_slice_free (FindURLsData, user_data);
-
+  g_list_free_full (urls, (GDestroyNotify)ephy_history_url_free);
   g_slist_free_full (list, (GDestroyNotify)free_potential_row);
+  g_clear_object (&priv->cancellable);
 }
 
 static void
@@ -534,16 +652,33 @@ ephy_completion_model_update_for_string (EphyCompletionModel *model,
   user_data->callback = callback;
   user_data->user_data = data;
 
+  if (priv->cancellable) {
+    g_cancellable_cancel (priv->cancellable);
+    g_object_unref (priv->cancellable);
+  }
+  priv->cancellable = g_cancellable_new ();
+
   ephy_history_service_find_urls (priv->history_service,
                                   0, 0,
                                   MAX_COMPLETION_HISTORY_URLS, 0,
-                                  query, NULL,
+                                  query,
+                                  EPHY_HISTORY_SORT_MOST_VISITED,
+                                  priv->cancellable,
                                   (EphyHistoryJobCallback)query_completed_cb,
                                   user_data);
 }
 
 EphyCompletionModel *
-ephy_completion_model_new (void)
+ephy_completion_model_new (EphyHistoryService *history_service,
+                           EphyBookmarks      *bookmarks,
+                           gboolean            use_markup)
 {
-  return g_object_new (EPHY_TYPE_COMPLETION_MODEL, NULL);
+  g_return_val_if_fail (EPHY_IS_HISTORY_SERVICE (history_service), NULL);
+  g_return_val_if_fail (EPHY_IS_BOOKMARKS (bookmarks), NULL);
+
+  return g_object_new (EPHY_TYPE_COMPLETION_MODEL,
+                       "history-service", history_service,
+                       "bookmarks", bookmarks,
+                       "use-markup", use_markup,
+                       NULL);
 }
